@@ -2,71 +2,80 @@ package Log::Dispatch::Config;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = '0.11';
+$VERSION = 0.12;
 
 use Log::Dispatch;
 use base qw(Log::Dispatch);
-use fields qw(config ctime);
+use fields qw(config);
 
 # caller depth: can be changed from outside
 $Log::Dispatch::Config::CallerDepth = 0;
 
+sub _croak { require Carp; Carp::croak(@_); }
+
+# accessor for symblic reference
+sub __instance {
+    my $class = shift;
+    no strict 'refs';
+    my $instance = "$class\::_instance";
+    $$instance = shift if @_;
+    return $$instance;
+}
+
 sub configure {
     my($class, $config) = @_;
-    die "no config file or configurator supplied" unless $config;
+    _croak "no config file or configurator supplied" unless $config;
 
     # default configurator: AppConfig
     unless (UNIVERSAL::isa($config, 'Log::Dispatch::Configurator')) {
 	require Log::Dispatch::Configurator::AppConfig;
 	$config = Log::Dispatch::Configurator::AppConfig->new($config);
     }
+    $config->myinit;
+    $class->__instance($config);
+}
 
-    no strict 'refs';
-    my $instance = "$class\::_instance";
-    $$instance = $config;
+sub configure_and_watch {
+    my($class, $config) = @_;
+    $class->configure($config);
+    $config = $class->__instance; # XXX: __instance is now config
+    $config->should_watch(1);	# tells conf to watch config file
 }
 
 # backward compatibility
-sub Log::Dispatch::instance {
-    __PACKAGE__->instance;
-}
+sub Log::Dispatch::instance { __PACKAGE__->instance; }
 
 sub instance {
     my $class = shift;
 
-    no strict 'refs';
-    my $instance = "$class\::_instance";
-    unless (defined $$instance) {
-	require Carp;
-	Carp::croak("Log::Dispatch::Config->configure not yet called.");
-    }
-
-    if (UNIVERSAL::isa($$instance, 'Log::Dispatch::Config')) {
+    my $instance = $class->__instance or _croak "Log::Dispatch::Config->configure not yet called.";
+    if (UNIVERSAL::isa($instance, 'Log::Dispatch::Config')) {
         # reload singleton on the fly
-	if ($$instance->needs_reload) {
-	    $$instance = $$instance->reload;
-	}
+	$class->reload if $instance->needs_reload;
     }
     else {
         # first time call: $_instance is L::D::Configurator::*
-	$$instance = $class->create_instance($$instance);
+	$class->__instance($class->create_instance($instance));
     }
-    return $$instance;
+    return $class->__instance;
 }
 
 sub needs_reload {
     my $self = shift;
-    return $self->{config}->needs_reload($self);
+    return $self->{config}->should_watch && $self->{config}->needs_reload;
 }
 
 sub reload {
-    my $self = shift;
-    my $class = ref $self;
-    return $class->create_instance($self->{config}->reload);
+    my $proto = shift;
+    my $class = ref $proto || $proto;
+    my $instance = $class->__instance;
+    $instance->{config}->reload;
+    $class->__instance($class->create_instance($instance->{config}));
 }
 
 sub create_instance {
     my($class, $config) = @_;
+    $config->{LDC_ctime} = time;	# creation time
 
     my $global = $config->get_attrs_global;
     my $callback = $class->format_to_cb($global->{format}, 0);
@@ -92,7 +101,6 @@ sub create_instance {
 
     # config info
     $instance->{config} = $config;
-    $instance->{ctime} = time;
 
     return $instance;
 }
@@ -176,14 +184,14 @@ Log::Dispatch::Config - Log4j for Perl
 =head1 SYNOPSIS
 
   use Log::Dispatch::Config;
-  Log::Dispatch::Config->configure('/path/to/config');
+  Log::Dispatch::Config->configure('/path/to/log.conf');
 
   my $dispatcher = Log::Dispatch::Config->instance;
   $dispatcher->debug('this is debug message');
   $dispatcher->emergency('something *bad* happened!');
 
-  # or the same
-  my $dispatcher = Log::Dispatch->instance;
+  # automatic reloading conf file, when modified
+  Log::Dispatch::Config->configure_and_watch('/path/to/log.conf');
 
   # or if you write your own config parser:
   use Log::Dispatch::Configurator::XMLSimple;
@@ -209,16 +217,6 @@ So, what you should do is call C<configure> method once in somewhere
 (like C<startup.pl> in mod_perl), then you can get configured
 dispatcher instance via C<Log::Dispatch::Config-E<gt>instance>.
 
-Formerly, C<configure> method declares C<instance> method in
-Log::Dispatch namespace. Now it inherits from Log::Dispatch, so the
-namespace pollution is not necessary. Currrent version still defines
-one-liner shortcut:
-
-  sub Log::Dispatch::instance { Log::Dispatch::Config->instance }
-
-so still you can call C<Log::Dispatch-E<gt>instance>, if you prefer,
-or for backward compatibility.
-
 =head1 CONFIGURATION
 
 Here is an example of the config file:
@@ -337,10 +335,36 @@ returning same object.
 
 See GoF Design Pattern book for Singleton Pattern.
 
-But in practice, in persistent environment like mod_perl, Singleton
-instance is not so useful. Log::Dispatch::Config defines C<instance>
-method so that the object reloads itself when configuration file is
-modified since its last object creation time.
+But in practice, in persistent environment like mod_perl, lifetime of
+Singleton instance becomes sometimes messy. If you want to reload
+singleton object manually, call C<reload> method.
+
+  Log::Dispatch::Config->reload;
+
+And, if you want to reload object on the fly, as you edit C<log.conf>
+or something like that, what you should do is to call
+C<configure_and_watch> method on Log::Dispatch::Config instead of
+C<configure>. Then C<instance> call will check mtime of configuration
+file, and compares it with instanciation time of singleton object. If
+config file is newer than last instanciation, it will automatically
+reload object.
+
+=head1 NAMESPACE COLLISION
+
+If you use Log::Dispatch::Config in multiple projects on the same perl
+interpreter (like mod_perl), namespace collision would be a
+problem. Bizzare thing will happen when you call
+C<Log::Dispatch::Config-E<gt>configure> multiple times with differenct
+argument.
+
+In such cases, what you should do is to define your own logger class.
+
+  package My::Logger;
+  use Log::Dispatch::Config;
+  use base qw(Log::Dispatch::Config);
+
+Or make wrapper for it. See L<POE::Component::Logger> implementation
+by Matt Sergeant.
 
 =head1 PLUGGABLE CONFIGURATOR
 
@@ -355,15 +379,17 @@ hardwired version equivalent to the one above in L</"CONFIGURATION">.
 
 =item *
 
-Inherit from Log::Dispatch::Configurator. Stub C<new> constructor is
-inherited, but you can roll your own with it.
+Inherit from Log::Dispatch::Configurator.
 
   package Log::Dispatch::Configurator::Hardwired;
   use base qw(Log::Dispatch::Configurator);
 
-  sub new {
-      bless {}, shift;
-  }
+Declare your own C<new> constructor. Stub C<new> method is defined in
+Configurator base class, but you want to put parsing method in your
+own constructor. In this example, we just bless reference. Note that
+your object should be blessed hash.
+
+  sub new { bless {}, shift }
 
 =item *
 
@@ -411,9 +437,9 @@ reference of parameters associated with the dispatcher.
 =item *
 
 Implement optional C<needs_reload> and C<reload>
-methods. C<needs_reload> accepts Log::Dispatch::Config instance and
-should return boolean value if the object is stale and needs reloading
-itself.
+methods. C<needs_reload> should return boolean value if the object is
+stale and needs reloading itself. This method will be triggered when
+you configure logging object with C<configure_and_watch> method.
 
 Stub config file mtime based C<needs_reload> method is declared in
 Log::Dispatch::Configurator as below, so if your config class is based
@@ -424,21 +450,15 @@ on filesystem files, you do not need to reimplement this.
       return $obj->{ctime} < (stat($self->{file}))[9];
   }
 
-If you do not need I<singleton-ness>, always return true.
+If you do not need I<singleton-ness> at all, always return true.
 
   sub needs_reload { 1 }
 
-C<reload> method is called when C<needs_reload> returns true, and
-should return new Configurator instance. Typically you should place
-configuration parsing again on this method, so
-Log::Dispatch::Configurator again declares stub C<reload> method that
-clones your object.
+C<reload> method should redo parsing of the config file. Configurator
+base class has a stub null C<reload> method, so you should better
+override it.
 
-  sub reload {
-      my $self = shift;
-      my $class = ref $self;
-      return $class->new($self->{file});
-  }
+See Log::Dispatch::Configurator::AppConfig source code for details.
 
 =item *
 
@@ -480,7 +500,7 @@ to increase the caller stack depth. The default value is 0.
       $Logger->$level($msg);
   }
 
-Note that your log caller's namespace should not begin with
+Note that your log caller's namespace should not match against
 C</^Log::Dispatch/>, which makes this module confusing.
 
 =head1 AUTHOR
@@ -493,6 +513,7 @@ it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<Log::Dispatch::Configurator::AppConfig>, L<Log::Dispatch>, L<AppConfig>
+L<Log::Dispatch::Configurator::AppConfig>, L<Log::Dispatch>,
+L<AppConfig>, L<POE::Component::Logger>
 
 =cut
