@@ -2,22 +2,30 @@ package Log::Dispatch::Configurator;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = 0.01;
+$VERSION = 0.12;
 
 sub new {
     my($class, $file) = @_;
-    return bless { file => $file }, $class;
+    bless { file   => $file }, $class;
 }
 
+sub myinit {
+    my $self = shift;
+    $self->{LDC_ctime} = 0 unless defined $self->{LDC_ctime};
+    $self->{LDC_watch} = 0 unless defined $self->{LDC_watch};
+}
+
+sub reload { }
+
 sub needs_reload {
-    my($self, $obj) = @_;
-    return $obj->{ctime} < (stat($self->{file}))[9];
+    my $self = shift;
+    return $self->{LDC_ctime} < (stat($self->{file}))[9];
 }
 
-sub reload {
+sub should_watch {
     my $self = shift;
-    my $class = ref $self;
-    return $class->new($self->{file});
+    $self->{LDC_watch} = shift if @_;
+    return $self->{LDC_watch};
 }
 
 sub _abstract_method {
@@ -45,8 +53,8 @@ Log::Dispatch::Configurator - Abstract Configurator class
   sub get_attrs        { }
 
   # optional
+  sub parse        { }
   sub needs_reload { }
-  sub reload       { }
 
 =head1 DESCRIPTION
 
