@@ -2,7 +2,7 @@ package Log::Dispatch::Configurator::AppConfig;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = 0.02;
+$VERSION = 0.12;
 
 use Log::Dispatch::Configurator;
 use base qw(Log::Dispatch::Configurator);
@@ -10,6 +10,13 @@ use AppConfig;
 
 sub new {
     my($class, $file) = @_;
+    my $self = bless { file => $file }, $class;
+    $self->parse_file;
+    return $self;
+}
+
+sub parse_file {
+    my $self = shift;
     my $config = AppConfig->new({
 	CREATE => 1,
 	GLOBAL => {
@@ -18,9 +25,14 @@ sub new {
     });
     $config->define(dispatchers => { DEFAULT => '' });
     $config->define(format      => { DEFAULT => undef });
-    $config->file($file);
+    $config->file($self->{file});
 
-    bless { file => $file, _config => $config }, $class;
+    $self->{_config} = $config;
+}
+
+sub reload {
+    my $self = shift;
+    $self->parse_file;
 }
 
 sub _config { $_[0]->{_config} }
